#ifndef DUCHAMP_DATA_ARRAY_BASE_H_
#define DUCHAMP_DATA_ARRAY_BASE_H_

#include <iostream>
#include <duchamp/duchamp.hh>

namespace duchamp
{
    template <class T> 
    class DataArrayBase
    {
    public:
	DataArrayBase();
	DataArrayBase(std::vector<size_t> dim);
	DataArrayBase(const DataArrayBase<T>& other);
	DataArrayBase<T>& operator= (const DataArrayBase<T>& other);
	virtual ~DataArrayBase();

	T *data(){return itsArray;};
	void setArray(T *arr){itsArray=arr;}; // share the storage
	void copyArray(T *arr); // deep copy of array

	std::vector<size_t> dim(){return itsDim;};
	size_t ndim(){return itsDim.size();};
	size_t size(){return itsNumPixels;};

	T operator()(size_t pix);
	T operator()(size_t *pix);
	T operator()(std::vector<size_t> &pix);

	void operator+=(T val);
	void operator+=(DataArrayBase<T> &other);
	void operator*=(T val);
	void operator*=(DataArrayBase<T> &other);
	void operator-=(T val);
	void operator-=(DataArrayBase<T> &other);
	void operator/=(T val);
	void operator/=(DataArrayBase<T> &other);

    protected:
    
	void setup();
    
	T *itsArray;
	std::vector<size_t> itsDim;
	size_t itsNumPixels;
    
    };

}

#include <duchamp/DataArrays/DataArrayBase.tcc>

#endif
